;; -*- Mode: Scheme -*-
;; Constraint definitions for Texas Instruments MSP430.
;; Copyright (C) 2011 Free Software Foundation, Inc.
;;
;; This file is part of GCC.
;;
;; GCC is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3, or (at your option)
;; any later version.
;;
;; GCC is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with GCC; see the file COPYING3.  If not see
;; <http://www.gnu.org/licenses/>.

;;; Available letters:
;;;    ABCD  GHIJKL  O          Z
;;;    abcdef h jkl    q  t v x  
;;; (nb: generic used are EFVXgimnoprs)

;; Note that the hard frame pointer is also a general register.

(define_register_constraint "z" "PC_REG"
  "Program counter r0")

(define_register_constraint "y" "SP_REG"
  "Stack pointer r1")

(define_register_constraint "Y" "GENERAL_SP_REGS"
  "Union of r (GENERAL), y (SP)")

(define_register_constraint "w" "SR_REG"
  "Status register r2")

(define_register_constraint "W" "GENERAL_SR_REGS"
  "Union of r (GENERAL), w (SR)")

(define_register_constraint "u" "FIXED_PSEUDO_REGS"
  "Fixed pseudo-register (argp, soft fp)")

(define_register_constraint "U" "FIXED_PSEUDO_GENERAL_SP_REGS"
  "Union of u (FIXED PSEUDO), r (GENERAL), y (SP)")

(define_constraint "R"
  "Indirect register @rN (general or stack pointer)"
  (match_test "msp430_indirect_register_operand (op)"))

(define_constraint "S"
  "Indirect stack pointer @r2"
  (match_test "msp430_indirect_register_operand (op) && REGNO (XEXP (op, 0)) == STACK_POINTER_REGNUM"))

(define_constraint "T"
  "Indexed stack pointer X(r2)"
  (match_test "msp430_indexed_register_operand (op) && REGNO (XEXP (XEXP (op, 0), 0)) == STACK_POINTER_REGNUM"))

(define_constraint "M"
  "CPUX and integer constant suitable for multi-position shift operations"
  (and (match_code "const_int")
       (match_test "msp430_cpu & MSP430_CPU_MSP430X && MSP430_CPUX_MULTISHIFT_COUNT_P(ival)")))

(define_constraint "Q"
  "Post-increment memory reference"
  (and (match_code "mem")
       (match_code "post_inc" "0")
       (match_code "reg" "00")))

(define_constraint "N"
  "General purpose register number as integer constant "
  (and (match_code "const_int")
       (match_test "(MSP430_MIN_GENERAL_REGNUM <= ival) && (ival <= MSP430_MAX_GENERAL_REGNUM)")))

(define_constraint "P"
  "Integer constant supported by constant generator registers"
  (and (match_code "const_int")
       (match_test "(ival == -1 || ival == 0 || ival == 1 || ival == 2 || ival == 4 || ival == 8)")))

