#include "config.h"
#include "system.h"
#include "prefix.h"
#include "defaults.h"

extern const char *msp430_mcucpp (int argc, const char **argv);

extern const char *msp430_mculdscriptpaths (int argc, const char **argv);

/* Return a new string -D__(toupper(mcu))__ */
static const char *
gen_cpp (const char *mcu)
{
  static const char prefix[] = "-D__";
  static const char suffix[] = "__";
  size_t len;
  char *result;
  char *ep;
  const char *cp;

  len = (sizeof (prefix) - 1) + strlen (mcu) + sizeof (suffix);
  result = XNEWVAR (char, len);
  ep = result;
  strcpy (ep, prefix);
  ep += sizeof (prefix) - 1;
  cp = mcu;
  while (*cp)
    {
      *ep++ = TOUPPER (*cp);
      ++cp;
    }
  *ep = 0;
  strcat (result, suffix);
  return result;
}

const char *
msp430_mcucpp (int argc, const char **argv)
{
  const char *mcu;
  const char *defs;
  const char *p;
#if 0
  fprintf (stderr, "%d args: ", argc);
  for (len = 0; len < argc; ++len)
    fprintf (stderr, "'%s' ", argv[len]);
  fprintf (stderr, "\n");
#endif

  if (0 == argc)
    return NULL;
  mcu = argv[argc - 1];
  defs = gen_cpp (mcu);
  p = strchr (mcu, '_');
  if (NULL != p)
    {
      char *base_mcu = xstrndup (mcu, p - mcu);
      defs = concat (gen_cpp (base_mcu), " ", defs, NULL);
      free (base_mcu);
    }
  return defs;
}

#ifndef DIR_UP
#define DIR_UP ".."
#endif /* DIR_UP */

/* Join additional directory components to a separator-terminated path
   to create a new separator-terminated path. */
static const char *
dir_join (const char *path, ...)
{
  static const char dir_sep[] = { DIR_SEPARATOR, 0 };
  const char *rv = path;
  const char *elt;
  va_list ap;

  va_start (ap, path);
  while (1)
    {
      elt = va_arg (ap, const char *);
      if (NULL == elt)
	break;
      rv = concat (rv, elt, dir_sep, NULL);
    }
  va_end (ap);
  return rv;
}

static const char *
gen_ldscriptpath (const char *mcu, const char *suffix)
{
  const char *gcc_exec_prefix;

  gcc_exec_prefix = getenv ("GCC_EXEC_PREFIX");
  if (!gcc_exec_prefix)
    gcc_exec_prefix = STANDARD_EXEC_PREFIX;

  return update_path (dir_join (gcc_exec_prefix, DIR_UP, DIR_UP,
				DEFAULT_TARGET_MACHINE, "lib",
				"ldscripts", mcu, suffix, NULL), "BINUTILS");
}

const char *
msp430_mculdscriptpaths (int argc, const char **argv)
{
  const char *mcu;
  const char *paths = "";
  const char *p;

  if (0 == argc)
    return 0;

  mcu = argv[argc - 1];
  p = strchr (mcu, '_');
  if (NULL != p)
    {
      mcu = xstrndup (mcu, p - mcu);
      paths = concat (paths, "-L ", gen_ldscriptpath (mcu, p + 1), NULL);
    }
  paths = concat (paths, " -L ", gen_ldscriptpath (mcu, NULL), NULL);
  return paths;
}
